@@ -951,13 +951,23 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 		}
 
 		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_COMP_LZO)) {
+			const char *v;
+
 			if (!args_params_check_nargs_minmax (params, 0, 1, &line_error))
 				goto handle_line_error;
-			if (!NM_IN_STRSET (params[1], NULL, "no", "yes", "adaptive")) {
+
+			v = params[1] ?: "adaptive";
+
+			if (nm_streq (v, "no")) {
+				/* old plasma-nm used to set "comp-lzo=no" to mean unset, thus it spoiled
+				 * to "no" option to be used in the connection. Workaround that, by instead
+				 * using "no-by-default" (bgo#769177). */
+				v = "no-by-default";
+			} else if (!NM_IN_STRSET (v, "yes", "adaptive")) {
 				line_error = g_strdup_printf (_("unsupported comp-lzo argument"));
 				goto handle_line_error;
 			}
-			setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO, params[1] ?: "adaptive");
+			setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO, v);
 			continue;
 		}
 
@@ -971,12 +981,21 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_RENEG_SEC)) {
 			if (!args_params_check_nargs_n (params, 1, &line_error))
 				goto handle_line_error;
-			if (!args_params_parse_int64 (params, 1, 0, 604800, &v_int64, &line_error))
+			if (!args_params_parse_int64 (params, 1, 0, G_MAXINT, &v_int64, &line_error))
 				goto handle_line_error;
 			setting_vpn_add_data_item_int64 (s_vpn, NM_OPENVPN_KEY_RENEG_SECONDS, v_int64);
 			continue;
 		}
 
+		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_MAX_ROUTES)) {
+			if (!args_params_check_nargs_n (params, 1, &line_error))
+				goto handle_line_error;
+			if (!args_params_parse_int64 (params, 1, 0, 100000000, &v_int64, &line_error))
+				goto handle_line_error;
+			setting_vpn_add_data_item_int64 (s_vpn, NM_OPENVPN_KEY_MAX_ROUTES, v_int64);
+			continue;
+		}
+
 		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_HTTP_PROXY_RETRY, NMV_OVPN_TAG_SOCKS_PROXY_RETRY)) {
 			if (!args_params_check_nargs_n (params, 0, &line_error))
 				goto handle_line_error;
@@ -1232,6 +1251,32 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			continue;
 		}
 
+		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_VERIFY_X509_NAME)) {
+			const char *type = "subject";
+			gs_free char *item = NULL;
+
+			if (!args_params_check_nargs_minmax (params, 1, 2, &line_error))
+				goto handle_line_error;
+			if (!args_params_check_arg_utf8 (params, 1, NULL, &line_error))
+				goto handle_line_error;
+
+			if (params[2]) {
+				if (!NM_IN_STRSET (params[2],
+				                   NM_OPENVPN_VERIFY_X509_NAME_TYPE_SUBJECT,
+				                   NM_OPENVPN_VERIFY_X509_NAME_TYPE_NAME,
+				                   NM_OPENVPN_VERIFY_X509_NAME_TYPE_NAME_PREFIX)) {
+					line_error = g_strdup_printf (_("invalid verify-x509-name type"));
+					goto handle_line_error;
+				}
+
+				type = params[2];
+			}
+
+			item = g_strdup_printf ("%s:%s", type, params[1]);
+			setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_VERIFY_X509_NAME, item);
+			continue;
+		}
+
 		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_REMOTE_CERT_TLS)) {
 			if (!args_params_check_nargs_n (params, 1, &line_error))
 				goto handle_line_error;
@@ -1809,6 +1854,8 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 
 	args_write_line_setting_value_int (f, NMV_OVPN_TAG_RENEG_SEC, s_vpn, NM_OPENVPN_KEY_RENEG_SECONDS);
 
+	args_write_line_setting_value_int (f, NMV_OVPN_TAG_MAX_ROUTES, s_vpn, NM_OPENVPN_KEY_MAX_ROUTES);
+
 	args_write_line_setting_value (f, NMV_OVPN_TAG_CIPHER, s_vpn, NM_OPENVPN_KEY_CIPHER);
 
 	args_write_line_setting_value (f, NMV_OVPN_TAG_TLS_CIPHER, s_vpn, NM_OPENVPN_KEY_TLS_CIPHER);
@@ -1816,8 +1863,11 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 	args_write_line_setting_value_int (f, NMV_OVPN_TAG_KEYSIZE, s_vpn, NM_OPENVPN_KEY_KEYSIZE);
 
 	value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO);
-	if (value)
+	if (value) {
+		if (nm_streq (value, "no-by-default"))
+			value = "no";
 		args_write_line (f, NMV_OVPN_TAG_COMP_LZO, value);
+	}
 
 	if (nm_streq0 (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_FLOAT), "yes"))
 		args_write_line (f, NMV_OVPN_TAG_FLOAT);
@@ -1867,17 +1917,34 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 	if (NM_IN_STRSET (connection_type,
 	                  NM_OPENVPN_CONTYPE_TLS,
 	                  NM_OPENVPN_CONTYPE_PASSWORD_TLS)) {
-		args_write_line_setting_value (f, NMV_OVPN_TAG_TLS_REMOTE, s_vpn, NM_OPENVPN_KEY_TLS_REMOTE);
+		const char *x509_name, *ta_key;
+
 		args_write_line_setting_value (f, NMV_OVPN_TAG_REMOTE_CERT_TLS, s_vpn, NM_OPENVPN_KEY_REMOTE_CERT_TLS);
 		args_write_line_setting_value (f, NMV_OVPN_TAG_NS_CERT_TYPE, s_vpn, NM_OPENVPN_KEY_NS_CERT_TYPE);
+		args_write_line_setting_value (f, NMV_OVPN_TAG_TLS_REMOTE, s_vpn, NM_OPENVPN_KEY_TLS_REMOTE);
 
-		value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TA);
-		if (_arg_is_set (value)) {
+		x509_name =  nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_VERIFY_X509_NAME);
+		if (_arg_is_set (x509_name)) {
+			const char *name;
+			gs_free char *type = NULL;
+
+			name = strchr (x509_name, ':');
+			if (name) {
+				type = g_strndup (x509_name, name - x509_name);
+				name++;
+			} else
+				name = x509_name;
+
+			args_write_line (f, NMV_OVPN_TAG_VERIFY_X509_NAME, name, type);
+		}
+
+		ta_key = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TA);
+		if (_arg_is_set (ta_key)) {
 			gs_free char *s_free = NULL;
 
 			args_write_line (f,
 			                 NMV_OVPN_TAG_TLS_AUTH,
-			                 nmv_utils_str_utf8safe_unescape_c (value, &s_free),
+			                 nmv_utils_str_utf8safe_unescape_c (ta_key, &s_free),
 			                 _arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TA_DIR)));
 		}
 	}
